#ifndef SHA_COMMON

#define SHA256_RESULTLEN (256 / 8)
#define SHA256_BLOCK_SIZE (512 / 8)

#define SHA384_RESULTLEN (384 / 8)
#define SHA384_BLOCK_SIZE (1024 / 8)

#define SHA512_RESULTLEN (512 / 8)
#define SHA512_BLOCK_SIZE (1024 / 8)

#endif
